import os
import time
import flask
import threading
from sspaf.core import render
import waitress

class ChangeDetector():
    def __init__(self, path: str):
        self.path = path
        self.last_modified = {}
        self.stop = False

    def run(self):
        previous_contents = set(os.listdir(self.path))
        while True:
            if self.stop:
                break
            current_contents = set(os.listdir(self.path))
            if current_contents != previous_contents:
                render.render(self.path)
                previous_contents = current_contents
            
            for filename in current_contents:
                file_path = os.path.join(self.path, filename)
                if os.path.isfile(file_path):
                    modification_time = os.path.getmtime(file_path)
                    if modification_time > self.last_modified.get(file_path, 0):
                        render.render(self.path)
                        self.last_modified[file_path] = modification_time
            
            time.sleep(1)

def serve(output_path: str) -> None:
    if output_path == ".":
        output_path = os.getcwd()

    print(f'Serving project {output_path}')

    app = flask.Flask(__name__, static_folder=os.path.join(output_path, 'output'))

    @app.route('/sspaf.js')
    def send_sspaf():
        return flask.send_file(output_path + '/output/sspaf.js')
    
    # jsons
    @app.route('/<path:path>.json')
    def send_json(path):
        return flask.send_file(output_path + '/output/' + path + '.json')

    @app.route('/')
    def send_index():
        return flask.send_file(output_path + '/output/index.html')

    @app.route('/<path:path>')
    def send_page(path):
        return flask.send_file(output_path + '/output/' + path)

    detector = ChangeDetector(output_path)
    thread = threading.Thread(target=detector.run)
    thread.start()

    render.render(output_path)
    print(f"Starting server on http://localhost:8080/")
    os.system("start http://localhost:8080/")
    waitress.serve(app, host='127.0.0.1', port=8080)
    detector.stop = True
    thread.join()


page = """
let pages = SSPAF_PAGES;

page_contents = {};

for (let i = 0; i < pages.length; i++) {
    fetch(pages[i]).then(response => response.json().then(data => {
        page_contents[data.title] = {
            "title": data.title,
            "content": data.content
        }
    }));
}

setTimeout(() => {
    updateContent();
}, 200);

window.addEventListener("hashchange", updateContent);

function updateContent() {
    let current_page = window.location.hash;
    console.log(current_page);
    current_page = current_page.replace("#", "");

    if (page_contents[current_page] != undefined) {
        document.title = page_contents[current_page].title;
        document.getElementById("content").innerHTML = page_contents[current_page].content;
    };
};
"""


import os
import time
from sspaf.core import render

def publish(path: str) -> None:
    if path == ".":
        path = os.getcwd()

    print(f'Publishing project {path}')
    start = time.time()

    render.render(path, dev=False)

    print(f'Published project {path} in {time.time() - start} seconds')
    print(f'Project published to {os.path.join(path, "output")}')


import os
import argparse

from sspaf.core import render, serve, publish, create

def args_error(a):

    print("""
(: SSPAF a superglued single page aooplication framework :)

Author: plusleft

Usage: sspaf <operation> <path>

Operations:
    render: render the project to the output folder
    serve: serve the project on port 8080
    publish: create a production ready version of the project
    create: initialize the blueprint for a new project

Path:
    path to the project

Examples:
    sspaf serve .
    sspaf publish /home/plusleft/myproject
    sspaf create /home/plusleft/myproject
""")
    
    exit(1)

def main():
    arguments = argparse.ArgumentParser()
    arguments.error = args_error
    arguments.add_argument(type=str, help='render, publish, create, serve', dest='operation')
    arguments.add_argument(type=str, help='path to the project', dest='path')

    args = arguments.parse_args()

    if args.operation == 'render':
        render.render(args.path)
    elif args.operation == 'serve':
        serve.serve(args.path)
    elif args.operation == 'publish':
        publish.publish(args.path)
    elif args.operation == 'create':
        create.create(args.path)
    else:
        print('Operation not found')

if __name__ == '__main__':
    main()


page = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>SSPAF_TITLE</title>
</head>
<script src="sspaf.js" defer></script>
<body>
    SSPAF_HEADER
    <div id="content" style="width: 100%; height: 100%;">SSPAF_INDEX</div>
    SSPAF_FOOTER
</body>
</html>
"""


import os
import time

def create(path: str) -> None:
    if path == ".":
        path = os.getcwd()

    print(f'Creating project {path}')
    start = time.time()

    os.makedirs(path, exist_ok=True)

    with open(os.path.join(path, 'index.html'), 'w+') as f:
        f.write(
"""
<h1>Hello, World!</h1>
""")
    
    with open(os.path.join(path, 'header.html'), 'w+') as f:
        f.write(
"""
<header>
    <a href="#index">Home</a>
</header>
""")
        
    with open(os.path.join(path, 'footer.html'), 'w+') as f:
        f.write(
"""
<footer>
    <p>Footer</p>
</footer>
""")


import os
import time
from sspaf.core.assets import init, js
import shutil

def render(path: str, dev=True) -> None:
    if path == ".":
        path = os.getcwd()

    start = time.time()

    # remove output folder
    shutil.rmtree(os.path.join(path, 'output'), ignore_errors=True)

    # create output folder
    os.makedirs(os.path.join(path, 'output'), exist_ok=True)

    # create the expected folder structure using os.walk
    for root, dirs, files in os.walk(path):
        for dir in dirs:
            if dir == 'output':
                continue
            os.makedirs(os.path.join(path, 'output', dir), exist_ok=True)

    # copy all the fiels from the main folder
    for file in os.listdir(path):
        if file.endswith('.html'):
            continue
        if shutil.os.path.isdir(os.path.join(path, file)):
            continue
        # TODO no
        try:
            shutil.copy(os.path.join(path, file), os.path.join(path, 'output', file))
        except shutil.SameFileError:
            pass

    # copy all non html files to the output folder and the expected folders
    for root, dirs, files in os.walk(path):
        for dir in dirs:
            if "output" in dir:
                continue
            for file in os.listdir(os.path.join(root, dir)):
                if file.endswith('.html'):
                    continue
                # TODO no
                try:
                    shutil.copy(os.path.join(root, dir, file), os.path.join(path, 'output', dir, file))
                except shutil.SameFileError:
                    pass

    # create the acording json files for all html files in the expected folders
    # main folder
    init_path(path, "")

    # subfolders
    for root, dirs, files in os.walk(path):
        for dir in dirs:
            if "output" in dir:
                continue
            init_path(path, dir)

    pages = []

    for file in os.listdir(os.path.join(path, 'output')):
        if file.endswith('.json'):
            pages.append(file)

    # loop over all the files in output dir and find all json files
    for root, dirs, files in os.walk(os.path.join(path, 'output')):
        for dir in dirs:
            for file in os.listdir(os.path.join(root, dir)):
                if file.endswith('.json'):
                    pages.append("/" + os.path.join(dir, file))

    # create the js file
    js_handle = open(os.path.join(path, 'output', 'sspaf.js'), "w+")
    js_content = js.page
    js_content = js_content.replace("SSPAF_PAGES", str(pages))
    js_handle.write(js_content)
    js_handle.close()


    end = time.time()
    print(f"The project was rendered in {end - start} seconds")


def init_path(root: str, path: str) -> None:
    # TODO no
    if "output" in root:
        return

    for file in os.listdir(os.path.join(root, path)):
        if file.endswith("header.html") or file.endswith("footer.html") or not file.endswith('.html'):
            continue

        html_handle = open(os.path.join(root, path, file), "r")
        page_content = html_handle.read().replace("\n", "").replace('"', '\\"')
        html_handle.close()

        json_handle = open(os.path.join(root, 'output', path, file.replace(".html", ".json")), "w+")
        if path == "":
            json_handle.write(f'{{"title": "{"/" + file.replace(".html", "")}", "content": "{page_content}"}}')
        else:
            json_handle.write(f'{{"title": "{"/" + path + "/" + file.replace(".html", "")}", "content": "{page_content}"}}')
        json_handle.close()

        if path == "":
            # create the index page for all the files
            init_page_content = init.page
            init_page_content = init_page_content.replace("SSPAF_TITLE", "index")
            page_content = open(os.path.join(root, path, "index.html"), "r").read()
            init_page_content = init_page_content.replace("SSPAF_INDEX", page_content)

            try:
                header_handle = open(os.path.join(root, path, "header.html"), "r")
                header_content = header_handle.read()
                header_handle.close()
            except FileNotFoundError:
                header_content = ""

            try:
                footer_handle = open(os.path.join(root, path, "footer.html"), "r")
                footer_content = footer_handle.read()
                footer_handle.close()
            except FileNotFoundError:
                footer_content = ""

            init_page_content = init_page_content.replace("SSPAF_HEADER", header_content)
            init_page_content = init_page_content.replace("SSPAF_FOOTER", footer_content)

            init_page_handle = open(os.path.join(root, 'output', path, "index.html"), "w+")
            init_page_handle.write(init_page_content)
            init_page_handle.close()


import setuptools

setuptools.setup(
    name="sspaf",
    version="0.0.11",
    description="A superglued single page application framework",
    author="plusleft",
    url="https://github.com/0xleft/sspaf",
    packages=setuptools.find_packages(),
    install_requires=[
        'flask',
        'waitress',
    ],
    entry_points={
        'console_scripts': [
            'sspaf = sspaf.sspaf_bin:main'
        ]
    }
)